<?php

namespace Teknisa\CRM\Controller;

use Teknisa\CRM\Controller\TeknisaCrud;
use Zeedhi\Framework\DataSource\ParameterBag;
use Teknisa\CRM\Controller\ReplicacaoParceiroController as Replicacao;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Teknisa\Model\Util\NovoCodigo;

class crmClienteController extends TeknisaCrud
{

    protected $dataSourceName = 'cliente_hierarquiaPerfilDeAcesso';
    protected $parameterBagColumns = array('P_NRPARCNEGOCIO', 'P_CRM_NRORGTRAB', 'IDTPOPERADOR', 'NRREVENDA', 'IDATIVO');

    protected $newCode;
    protected $environment;
    protected $entityManager;
    protected $connection;
    protected $clienteService;
    protected $parceiroService;

    public function __construct($crudService, $parameterBag, $entityManager, $newCode, $environment, $clienteService, $parceiroService, Replicacao $replicacao, $controlerNovaTarefa)
    {
        $this->connection                 = $entityManager->getConnection();
        $this->newCode                    = $newCode;
        $this->entityManager              = $entityManager;
        $this->environment                = $environment;
        $this->clienteService             = $clienteService;
        $this->replicacao                 = $replicacao;
        $this->controlerNovaTarefa        = $controlerNovaTarefa;
        $this->connection                 = $entityManager->getConnection();

        parent::__construct($crudService, $parameterBag);
    }

    public function save(DTO\Request $request, DTO\Response $response)
    {
        try {
            $this->connection->beginTransaction();

            foreach ($request->getDataSet()->getRows() as &$row) {
                if ($row['__is_new']) {
                    if ($row['IDTPIJURCLIE'] === 'F') {
                        $row['TIPOINSC'] = 'CPF';
                    } elseif ($row['IDTPIJURCLIE'] === 'J') {
                        $row['TIPOINSC'] = 'CNPJ';
                    } else {
                        $row['TIPOINSC'] = 'LIVRE';
                    }
                    $row['NRPARCRESP'] = $this->clienteService->getCdOperador($row['CDOPERRESPONSAVEL'], 'PARCNEGOCIO');
                    $row["CDTIPOPARCEIRO"] = "CLIENTE";
                    $row['NMFANTCLIE'] = empty($row['NMFANTCLIE']) ? $row['NMRAZSOCCLIE'] : $row['NMFANTCLIE'];
                    $row['IDLIGEXTCLIE'] = 'S';
                    $row['IDPRIPUBCLIE'] = 'V';
                    $parcCriado = $this->clienteService->verificaParceiro($row, $row["CDTIPOPARCEIRO"]);
                    if (empty($parcCriado)) {
                        $row['CDCLIENTE'] = $this->newCode->getCodeByTableName("CLIENTE", $row["NRORG"]);
                        $row['NRPARCNEGOCIO'] = $this->newCode->getCodeByTableName("PARCNEGOCIO", $row["NRORG"]);
                        $row['NRSEQCONTATO'] = $this->newCode->getCodeByTableName("NRSEQCONTATO", $row["NRORG"]);
                        $row['NRPARCERIA'] = NovoCodigo::geraCodigo($row["NRORG"], "PARCERIA", 1, 10, false, false);
                        $row['CDOPERINCLUSAO'] = isset($row['CDOPEREVOL']) ? $row['CDOPEREVOL'] : $this->environment->getCdOperador();
                        $row['NRCONTATO'] = $row['NRSEQCONTATO'];
                        $row['CDFILIAL'] = $this->environment->getCdFilial();
                        $row['NRORGINCLUSAO'] = $row['NRORG'];
                        $row['NRORGULTATU'] = $row['NRORG'];
                        $row['CDOPERULTATU'] = $this->environment->getCdOperador();

                        $edited = false;
                    } else {
                        $edited = true;
                        $row['__is_new'] = false;
                    }

                    $row['NRSEQTPATIVCLI'] = $row['NRSEQTPATIVCLI'] == '' ? null : $row['NRSEQTPATIVCLI'];
                    $row['ISCPF'] = ($row['IDTPIJURCLIE'] === 'F') ? 'S' : 'N';

                    $this->clienteService->setParceiroNegocio($row, !empty($parcCriado));
                    if (empty($parcCriado)) {
                        $this->clienteService->newContato($row);
                        $this->clienteService->createPartnership($row);
                    }
                } else {
                    $edited = true;
                }
                $verica = self::verificaModoRecebInfo($row);

                if ($verica[0]["QUANTIDADE"] <= 0) {
                    self::setModoRecebInfo($row);
                } else {
                    self::updateModoRecebInfo($row);
                }
            }
            if (!$edited) {
                parent::save($request, $response);

                if (!empty($row['CDOPERRESPONSAVEL'])) {
                    $this->updateResponsavel($row);
                }

                $error = $response->getError();
                if (!empty($error)) {
                    throw new \Exception($error->getMessage(), $error->getErrorCode());
                }
            } else {
                $this->clienteService->updateCliente($row);
                if (!empty($row['CDOPERRESPONSAVEL'])) {
                    $this->updateResponsavel($row);
                }
                self::updateTipoAtividade($row);
                if (!empty($row['CDOPERRESPONSAVEL'])) {
                    $this->updateResponsavel($row);
                }
            }
            $replicacao = $this->replicacao->replica($row['NRPARCNEGOCIO'], 'CLIENTE', 'CLIENTE');
            $response->addDataSet(new DataSet("REPLICACAO", array($replicacao)));
            $response->addDataSet(new DataSet("DADOS", array("CLIENTE" => $row['CDCLIENTE'], "NRPARCNEGOCIO" => $row['NRPARCNEGOCIO'])));

            $this->connection->commit();
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            $this->connection->rollBack();
        }
    }

    public function verificaTarefas(Row $row, Response $response)
    {
        $row = $row->getRow();
        //$row['CDOPERADORLOGADO'] = $this->environment->getCdOperador();
        $retorno = $this->clienteService->verificaTarefas($row);
        $tarefas = array();
        if (is_array($retorno)) {
            foreach ($retorno as $tarefa) {
                $tarefa['NMOPERADOR'] = $this->clienteService->getOperador($tarefa['CDOPERREAL'], $row['NRORG']);
                if (!(in_array($tarefa['NMOPERADOR'], $tarefas)) && !empty($tarefa['NMOPERADOR']))
                    array_push($tarefas, $tarefa['NMOPERADOR']);
            }
        }
        if (empty($tarefas)) {
            $tarefas = false;
        }
        $response->addDataSet(new DataSet("DADOS", array("TAREFAS" => $tarefas)));
    }

    public function procedureInsertNextTaskProspect(Row $row, Response $response)
    {
        $row = $row->getRow();
        $retorno = $this->clienteService->prepareInsertProspectEvolutionNextTask($row);

        $response->addDataSet(new DataSet("DADOS", array("NRSEQFILACONTA" => $retorno)));

        if ($row["CDOPEREVOL"] != $row["CDOPERREAL"] && !empty($row["DSEMAILOPER"])) {

            $paramsEmail = array(
                "TIPO"                  => "",
                "NMOPERREAL"            => empty($row["NMOPERADORREAL"]) ? '' : $row["NMOPERADORREAL"],
                "OPERADORDOLOGADO"      => empty($row["NMOPERADOR"]) ? '' : $row["NMOPERADOR"],
                "DSTIPO"                => null,
                "DESCRICAO"             => null,
                "NRSEQCONTATO"          => null,
                "DSFILACONTATO"         => empty($row["DSFILACONTATO"]) ? '' : $row["DSFILACONTATO"],
                "NMPARCNEGOCIO"         => empty($row["NMPARCNEGOCIO"]) ? '' : $row["NMPARCNEGOCIO"],
                "DTPLANFILACONTA"       => empty($row["DTPLANFILACONTA"]) ? '' : $row["DTPLANFILACONTA"],
                "DTVENCFILACONTA"       => null,
                "DSCLASSIFICATAREFA"    => null,
                "DSDETEFILACONTA"       => null,
                "DSEMAILOPER"           => empty($row["DSEMAILOPER"]) ? '' : $row["DSEMAILOPER"]
            );

            $this->controlerNovaTarefa->enviaEmailTarefa($paramsEmail);
        }
    }

    public function selectContatoProspect(Row $row, Response $response)
    {
        $row = $row->getRow();

        $retorno = $this->clienteService->prepareSelectProspectContact($row);
        $response->addDataSet(new DataSet("DADOS", $retorno));
    }

    public function updateTipoAtividade($row)
    {
        $updateCliente = "UPDATE PARCNEGOCIO
                         SET NRSEQTPATIVCLI = :NRSEQTPATIVCLI
                         WHERE NRPARCNEGOCIO = :NRPARCNEGOCIO AND
                         NRORG = :NRORG";

        $params = array(
            "NRSEQTPATIVCLI"    => $row['NRSEQTPATIVCLI'],
            "NRPARCNEGOCIO"     => $row['NRPARCNEGOCIO'],
            "NRORG"             => $row['NRORG']
        );

        $std = $this->connection->prepare($updateCliente);
        $std->execute($params);
    }

    public function insertNrseqcontatoForNewOpportunity(Row $row)
    {

        $row = $row->getRow();

        $this->clienteService->callForInsertNrseqcontatoForNewOpportunity($row);
    }

    public function updatePartnerType(Row $row)
    {

        $row = $row->getRow();

        $this->clienteService->callForUpdatePartnerType($row);
    }

    public function insertPartnershipOfTypeClient(Row $row)
    {

        $row = $row->getRow();

        $this->clienteService->callForInsertPartnershipOfTypeClient($row);
    }

    public function deactivateProspect(Row $row)
    {

        $row = $row->getRow();

        $this->clienteService->callForDeactivateProspect($row);
    }

    public function insertClientAddress(Row $row)
    {

        $row = $row->getRow();

        $this->clienteService->callForInsertClientAddress($row);
    }

    public function selectProspectAddress(Row $row, Response $response)
    {

        $row = $row->getRow();

        $returnedAddress = $this->clienteService->callForSelectAddressProspect($row);

        $response->addDataSet(new DataSet("DADOS", array("ADDRESS" => $returnedAddress)));
    }

    public function setTaskEndDate(Row $row)
    {

        $row = $row->getRow();

        $this->clienteService->callForSetTaskEndDate($row);
    }

    public function deactivateProspectTask(Row $row)
    {

        $row = $row->getRow();

        $taskUpdateParams = array(

            "NRSEQFILACONTA"    => $row["NRSEQFILACONTA"]

        );

        $this->clienteService->callForDeactivateProspectTask($taskUpdateParams);
    }

    public function convertInClient(Row $row, DTO\Response $response)
    {
        try {
            $row = $row->getRow();
            $nrparcnegocio = $this->parceiroService->replicaParceiro($row);

            if ($row['__is_new']) {
                $response->addDataset(new DataSet('columnsToReturn', array("NRPARCNEGOCIO" => $nrparcnegocio)));
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function replicaInterlocutorProspect(Row $row, DTO\Response $response)
    {
        $row = $row->getRow();

        if (!$row['__is_new']) {
            $this->clienteService->replicaInterlocutorProspect($row);
        }
    }

    public function setIdQualificadoProspect(Row $row)
    {
        $row = $row->getRow();

        $this->clienteService->setIdQualificadoProspect($row);
    }

    public function validaCNPJCliente(Row $row, Response $response)
    {

        $row = $row->getRow();

        $returnedClientes = $this->clienteService->validaCNPJCliente($row);

        $response->addDataSet(new DataSet("cnpjcliente", array("valid" => $returnedClientes)));
    }

    public function setModoRecebInfo($row)
    {
        $modoRecebeInfo = $row["NRMODORECEBINFO"] == '' ?
            null : $row["NRMODORECEBINFO"];

        $insertModo = "INSERT INTO GER_PARCNEGOCIOINFO (NRPARCNEGOCIO, NRORG, NRMODORECEBINFO)
         VALUES (:NRPARCNEGOCIO,
                 :NRORG,
                 :NRMODORECEBINFO)";

        $params = array(
            "NRPARCNEGOCIO"         => (int)$row["NRPARCNEGOCIO"],
            "NRORG"                 => $row["NRORG"],
            "NRMODORECEBINFO"       => $row["NRMODORECEBINFO"]
        );

        $std = $this->entityManager->getConnection()->prepare($insertModo);
        $std->execute($params);
    }

    public function updateModoRecebInfo($row)
    {


        $updatemodorecebeinfo = "UPDATE GER_PARCNEGOCIOINFO
                                 SET NRMODORECEBINFO = :NRMODORECEBINFO
                                 WHERE NRPARCNEGOCIO = :NRPARCNEGOCIO AND
                                 NRORG = :NRORG";
        $params = array(
            "NRMODORECEBINFO" => $row['NRMODORECEBINFO'],
            "NRPARCNEGOCIO" => $row['NRPARCNEGOCIO'],
            "NRORG" => $row['NRORG']
        );

        $std = $this->connection->prepare($updatemodorecebeinfo);
        $std->execute($params);
    }

    public function verificaModoRecebInfo($row)
    {

        $verificamodorecebinfo = "SELECT COUNT(NRPARCNEGOCIO) QUANTIDADE FROM GER_PARCNEGOCIOINFO where NRPARCNEGOCIO  =  :NRPARCNEGOCIO";

        $params = array(
            "NRPARCNEGOCIO" => $row['NRPARCNEGOCIO'],

        );

        $std = $this->connection->prepare($verificamodorecebinfo);
        $std->execute($params);
        $retornoModoRecebInfo = $std->fetchAll();

        return $retornoModoRecebInfo;
    }

    public function updateResponsavel($row)
    {
        try {
            $getOperador = "SELECT CDOPERADOR, NRPARCNEGOCIO FROM OPERADOR WHERE CDOPERADOR = :CDOPERADOR AND NRORG = :NRORG AND ROWNUM = 1";
            $paramsOper = array(
                "CDOPERADOR" => $row['CDOPERRESPONSAVEL'],
                "NRORG" => $row['NRORG']
            );

            $std1 = $this->connection->prepare($getOperador);
            $std1->execute($paramsOper);
            $operador = $std1->fetchAll();

            if (!empty($operador)) {
                $updateProspect = "UPDATE CRM_STATUSPARC
                                   SET NRPARCRESP = :NRPARCRESP,
                                       CDOPERRESP = :CDOPERRESP
                                   WHERE NRPARCNEGOCIO = :NRPARCNEGOCIO 
                                         AND NRORG = :NRORG
                                         AND CRM_NRORGTRAB = :CRM_NRORGTRAB";

                $params = array(
                    "NRORG"             => $row['NRORG'],
                    "CRM_NRORGTRAB"     => $row['CRM_NRORGTRAB'],
                    "NRPARCNEGOCIO"     => $row['NRPARCNEGOCIO'],
                    "NRPARCRESP"        => $operador[0]['NRPARCNEGOCIO'],
                    "CDOPERRESP"        => $operador[0]['CDOPERADOR']
                );

                $std = $this->connection->prepare($updateProspect);
                $std->execute($params);
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }
}
